//+------------------------------------------------------------------+
//|                                                    TestTimer.mq5 |
//|                                           Copyright 2010, A.Saku |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, A.Saku"
#property link      "http://www.mql5.com"
#property version   "1.00"

#include <trade/trade.mqh>

#define MagicNumber  1296
//+------------------------------------------------------------------+
//| Input                                                            |
//+------------------------------------------------------------------+
input long TradeInterval = 3600;  // Trade Interval (Second)


//+------------------------------------------------------------------+
//| Variables                                                        |
//+------------------------------------------------------------------+
datetime lastTime=0;
datetime dealTime=0;
int state=0;
double positionSize;
ulong tktLimitOrder, tktStopOrder;
ulong lastCommand;
double pips;

CTrade trade;   // Trade class

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
    lastTime=TimeCurrent();
    state=0;
    trade.SetExpertMagicNumber(MagicNumber);

    if (_Digits>3) pips = 0.0001; else pips=0.01;
//---
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
    if (PositionSelect(_Symbol)) {
        if (PositionGetDouble(POSITION_VOLUME)>0) trade.PositionClose(_Symbol);
    }
    if (tktStopOrder>0)  trade.OrderDelete (tktStopOrder);
    if (tktLimitOrder>0) trade.OrderDelete(tktLimitOrder);

//---
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
     bool result;
     MqlTick tick;

     if (TimeCurrent()-lastTime<TradeInterval) return;
     lastTime=TimeCurrent();

     lastCommand=-1;
     SymbolInfoTick(_Symbol,tick);

     if (state>7) return;

     //----------------------------------------------------------------
     // 1st stage open buy order
     //----------------------------------------------------------------
     if (state==0) {
        Print( "<<<<<<<<<< Stage1 : Try to deal market order >>>>>>>>>>" );

        result = trade.PositionOpen(_Symbol, ORDER_TYPE_BUY, 0.1, tick.ask, 0.0, 0.0, "Makert buy" );
        if (result && trade.ResultRetcode()==TRADE_RETCODE_DONE) {
           state++;
           lastCommand = TRADE_ACTION_DEAL;
           dealTime = lastTime;
           Print( ">>>>> Deal Ticket in the MqlTradeResult (should be \">0\") =", trade.ResultDeal());
        }
     }
     //----------------------------------------------------------------
     // 2nd stage place pending order
     //----------------------------------------------------------------
     else if ((state % 3)==1) {
        result=true;
        Print( "<<<<<<<<<< Stage2 : Try to place pending orders >>>>>>>>>>" );

        if (!trade.OrderOpen(_Symbol, ORDER_TYPE_SELL_LIMIT, 0.1, 0.0, tick.bid+600*pips, 0.0, 0.0, ORDER_TIME_GTC, 0, "Sell limit")) result=false;
        else tktLimitOrder = trade.ResultOrder();

        if (!trade.OrderOpen(_Symbol, ORDER_TYPE_SELL_STOP,  0.1, 0.0, tick.bid-600*pips, 0.0, 0.0, ORDER_TIME_GTC, 0, "Sell Stop"))  result=false;
        else tktStopOrder = trade.ResultOrder();

        if (result) {
            state++;
            lastCommand = TRADE_ACTION_PENDING;
            Print(">>>>> Stop Order Ticket=", tktStopOrder, "  Limit Order Ticket=", tktLimitOrder );
        }
    }
    //----------------------------------------------------------------
    // 3rd stage modify pending order
    //----------------------------------------------------------------
    else if ((state % 3)==2) {
        result=true;
        Print( "<<<<<<<<<< Stage3 : Try to modify pending order >>>>>>>>>>" );

        if (tktLimitOrder<=0 || !trade.OrderModify(tktLimitOrder, tick.bid+600*pips, 0.0, 0.0, ORDER_TIME_GTC, 0)) result=false;
        else Print( ">>>>> limit Order Ticket in the MqlTradeResult (should be \">0\") =", trade.ResultOrder());

        if (tktStopOrder<=0  || !trade.OrderModify(tktStopOrder,  tick.bid-600*pips, 0.0, 0.0, ORDER_TIME_GTC, 0))  result=false;
        else Print( ">>>>> stop  Order Ticket in the MqlTradeResult (should be \">0\") =", trade.ResultOrder());

        if (result) {
            state++;
            lastCommand = TRADE_ACTION_MODIFY;
        }
    }
    //----------------------------------------------------------------
    // 4th stage delete pending order
    //----------------------------------------------------------------
    else {
        result=true;
        Print( "<<<<<<<<<< Stage4 : Try to modify pending order >>>>>>>>>>" );

        if (tktLimitOrder<=0 || !trade.OrderDelete(tktLimitOrder)) result=false;
        if (tktStopOrder<=0  || !trade.OrderDelete(tktStopOrder))  result=false;
        if (result) {
           state++;
           lastCommand = TRADE_ACTION_REMOVE;
        }
     }
}


//+------------------------------------------------------------------+
//| Expert trade function                                            |
//+------------------------------------------------------------------+
void OnTrade()
{
    if (!IsCommandDone()) return;    // Command not completed

    HistorySelect(dealTime, TimeCurrent());
    int HistoryTotalNumber = HistoryDealsTotal();
    Print("/////////////////////////////////////////////////////////////////////");
    Print("//////  Total Deal Number=", HistoryTotalNumber);

    for (int i=0; i<HistoryTotalNumber; i++) {
        if (i>0) Print("////// --------------------------------------------------------------");

        ulong tkt = HistoryDealGetTicket(i);
        Print("//////  Deal#",i,"   Deal Ticket=", tkt);
        Print("//////  InitialLot=", HistoryDealGetDouble(tkt,DEAL_VOLUME));
        Print("//////  Price=",HistoryDealGetDouble(tkt,DEAL_PRICE));
        Print("//////  Time=", TimeToString( HistoryDealGetInteger(tkt,DEAL_TIME) ));
    }
    Print("/////////////////////////////////////////////////////////////////////");
    Print("");
}


//+------------------------------------------------------------------+
//| Check last command is complete                                   |
//+------------------------------------------------------------------+
bool IsCommandDone()
{
    // Because of MT5 tester bugs, this function is not working correctly
    if (MQL5InfoInteger(MQL5_TESTING)) return(true);


    // Last command is market buy
    if (lastCommand==TRADE_ACTION_DEAL) {
        ulong tkt = trade.ResultDeal();
        if (tkt>0) {
            HistorySelect(lastTime, TimeCurrent());
            for (int i=HistoryDealsTotal()-1; i>=0; i--) {
                if (tkt==HistoryDealGetTicket(i)) return(true);
            }
        }
        return(false);
    }

    // Last command is place pending order
    if (lastCommand==TRADE_ACTION_PENDING || lastCommand==TRADE_ACTION_MODIFY) {
        if (OrderSelect(trade.ResultOrder())) {
            if (OrderGetInteger(ORDER_STATE)==ORDER_STATE_PLACED) return(true);
        }
        return(false);
    }

    // Last command is remove pending order
    if (lastCommand==TRADE_ACTION_REMOVE) {
        HistorySelect(lastTime, TimeCurrent());
        for (int i=HistoryOrdersTotal()-1; i>=0; i--) {
            if (tktStopOrder==HistoryOrderGetTicket(i)) return(true);
        }
        return(false);
    }

    return(false);
}
